"""
    Persalys --- An OpenTURNS module
    ==================================

    Contents
    --------
      'Persalys' is a module for OpenTURNS

"""

# flake8: noqa

# ensures swig type tables order & dll load
import openturns as _ot

from .persalys import *

__version__ = "19.1"
